from .text_utils import *
from .audio_utils import *
from .seo_utils import *
from .functions import *

# Clear root logger handlers to prevent duplicate console output
logging.getLogger('').handlers.clear()

def transcribe_all_video_paths(directory=None,
                               output_dir=None,
                               remove_phrases=None,
                               summarizer=None,
                               kw_model=None,
                               get_vid_data=None,
                               uploader=None,
                               domain=None,
                               categories=None,
                               video_url=None):
    logger.info(f"Entering transcribe_all_video_paths")
    get_vid_data = get_vid_data or False
    remove_phrases = remove_phrases or []
    directory = directory or os.getcwd()
    output_dir = output_dir if output_dir else make_dirs(directory, 'text_dir')
    paths = glob.glob(path_join(directory, '**', '**'), recursive=True)
    paths = [file_path for file_path in paths if confirm_type(file_path, media_types=get_media_types(['video']))]
    video_paths = get_all_file_types(directory=directory, types='video') or get_all_file_types(directory=abs_dirname, types='videos')
    for video_path in video_paths:
        logger.info(f"Processing video: {video_path}")
        info = get_info_data(video_path, output_dir=output_dir, remove_phrases=remove_phrases)
        if not os.path.isfile(info['audio_path']):
            extract_audio_from_video(video_path=info['video_path'], audio_path=info['audio_path'])
        info = transcribe_audio_file(audio_path=info['audio_path'], json_data=info)
        info = get_text_and_keywords(info,summarizer=summarizer,kw_model=kw_model)
        info = get_seo_data(info,
                            uploader=uploader,
                            domain=domain,
                            categories=categories,
                            video_url=video_url)
        safe_dump_to_file(data=info, file_path=info['info_path'])
    logger.info(f"Exiting transcribe_all_video_paths")

def get_info_data(video_path, output_dir=None, remove_phrases=None):
    remove_phrases = remove_phrases or []
    dirname = os.path.dirname(video_path)
    basename = os.path.basename(video_path)
    filename, ext = os.path.splitext(basename)
    video_directory = make_dirs(output_dir, filename)
    info_path = os.path.join(video_directory, 'info.json')
    video_text_path = os.path.join(video_directory, 'video_text.json')
    audio_path = os.path.join(video_directory, 'audio.wav')
    video_json_path = os.path.join(video_directory, 'video_json.json')
    info = {}
    if os.path.isfile(info_path):
        info = safe_read_from_json(info_path)
    info['video_path'] = video_path
    info['video_directory'] = video_directory
    info['info_path'] = info_path
    info['filename'] = filename
    info['ext'] = ext
    info['remove_phrases'] = remove_phrases
    info['audio_path'] = audio_path
    info['video_json'] = video_json_path
    safe_dump_to_file(data=info, file_path=info['info_path'])
    return info




from .text_utils import *
from .audio_utils import *
from .functions import (logger,
                        create_key_value,
                        os,
                        timestamp_to_milliseconds,
                        format_timestamp,
                        get_time_now_iso,
                        parse_timestamp,
                        url_join)
from pydub import AudioSegment
from moviepy.editor import VideoFileClip
def update_sitemap(video_data, sitemap_path):
    with open(sitemap_path, 'a') as f:
        f.write(f"""
<url>
    <loc>{video_data['canonical_url']}</loc>
    <video:video>
        <video:title>{video_data['seo_title']}</video:title>
        <video:description>{video_data['seo_description']}</video:description>
        <video:thumbnail_loc>{video_data['thumbnail']['file_path']}</video:thumbnail_loc>
        <video:content_loc>{video_data['video_path']}</video:content_loc>
    </video:video>
</url>
""")
def pick_optimal_thumbnail(video_text, combined_keywords):
    scores = []

    for entry in video_text:
        text = entry['text'].lower()
        keyword_score = sum(1 for kw in combined_keywords if kw.lower() in text)
        clarity_score = 1 if len(text.strip()) > 20 else 0  # basic clarity check
        end_phrase_penalty = -1 if "thanks for watching" in text else 0
        total_score = keyword_score + clarity_score + end_phrase_penalty

        scores.append((entry['frame'], total_score, text.strip()))

    scores = sorted(scores, key=lambda x: x[1], reverse=True)
    return scores[0] if scores else None
def get_seo_data(info,uploader=None,domain=None,categories=None,video_url=None):
    categories = categories or {'ai': 'Technology', 'cannabis': 'Health', 'elon musk': 'Business'}
    uploader = uploader or 'The Daily Dialectics'
    domain = domain or 'https://thedailydialectics.com'
    video_url = video_url or url_join(domain,videos)
    primary_keyword = info['combined_keywords'][0] if info['combined_keywords'] else info['filename']
    info['seo_title'] = f"{primary_keyword} - {info['filename']}"[:70]
    summary = info.get('summary', 'No summary available.')
    keywords_str = ', '.join(info['combined_keywords'][:3])
    info['seo_description'] = f"{summary[:150]} Explore {keywords_str}. Visit thedailydialectics.com for more!"[:300]
    info['seo_tags'] = [kw for kw in info['combined_keywords'] if kw.lower() not in ['video', 'audio', 'file']]
    video_text = info.get('video_text')
    info['thumbnail']={}
    if video_text and len(video_text)>0:
        thumnail_data = video_text[0]
        info['thumbnail']['file_path']= os.path.join(info['thumbnails_dir'],thumnail_data.get("frame"))
        info['thumbnail']['alt_text']= thumnail_data.get("text")
    thumbnail_score   = pick_optimal_thumbnail(info['video_text'], info['combined_keywords'])
    if thumbnail_score:
        best_frame, score, matched_text = thumbnail_score
        info['thumbnail']['file_path']= os.path.join(info['thumbnails_dir'], best_frame)
        info['thumbnail']['alt_text']= matched_text[:100] 
    audio = AudioSegment.from_wav(info['audio_path'])
    info['duration_seconds'] = len(audio) / 1000
    info['duration_formatted'] = format_timestamp(len(audio))
    export_srt(info['audio_text'], os.path.join(info['video_directory'],"captions.srt"))
    info['captions_path'] = os.path.join(info['video_directory'],"captions.srt")
    info['schema_markup'] = {
        "@context": "https://schema.org",
        "@type": "VideoObject",
        "name": info['seo_title'],
        "description": info['seo_description'],
        "thumbnailUrl": info['thumbnail']['file_path'],
        "duration": f"PT{int(info['duration_seconds'] // 60)}M{int(info['duration_seconds'] % 60)}S",
        "uploadDate": get_time_now_iso(),
        "contentUrl": info['video_path'],
        "keywords": info['seo_tags']
    }
    info['social_metadata'] = {
        "og:title": info['seo_title'],
        "og:description": info['seo_description'],
        "og:image": info['thumbnail']['file_path'],
        "og:video": info['video_path'],
        "twitter:card": "player",
        "twitter:title": info['seo_title'],
        "twitter:description": info['seo_description'],
        "twitter:image": info['thumbnail']['file_path']
    }
    
    info['category'] = next((v for k, v in categories.items() if k in ' '.join(info['seo_tags']).lower()), 'General')
    info['uploader'] = {"name": uploader, "url": domain}
    info['publication_date'] = get_time_now_iso()
    video = mp.VideoFileClip(info['video_path'])
    info['file_metadata'] = {
        'resolution': f"{video.w}x{video.h}",
        'format': 'MP4',
        'file_size_mb': os.path.getsize(info['video_path']) / (1024 * 1024)
    }
    video.close()
    video_id = info['filename'].replace(' ', '-').lower()
    info['canonical_url'] = url_join(video_url,video_id)
    update_sitemap(info, f"{info['video_directory']}/../sitemap.xml")
    return info



from .functions import (logger,
                        create_key_value,
                        timestamp_to_milliseconds,
                        format_timestamp,
                        parse_timestamp,
                        get_time_now_iso,
                        shutil,
                        os,
                        safe_dump_to_file)
from moviepy.editor import *
import moviepy.editor as mp
import speech_recognition as sr
from pydub import AudioSegment
r = sr.Recognizer()
def check_if_transcribed_block(audio_text, start_time, end_time):
    for entry in audio_text:
        entry_start = timestamp_to_milliseconds(entry.get("start_time", "0:00"))
        entry_end = timestamp_to_milliseconds(entry.get("end_time", "0:00"))
        if entry_start <= start_time and entry_end >= end_time and entry.get("text", "").strip():
            return True
    uncovered_start, uncovered_end = start_time, end_time
    for entry in audio_text:
        entry_start = parse_timestamp(entry.get("start_time", "0:00"))
        entry_end = parse_timestamp(entry.get("end_time", "0:00"))
        if not entry.get("text", "").strip():
            continue
        if entry_start <= uncovered_start < entry_end:
            uncovered_start = entry_end
        if entry_start < uncovered_end <= entry_end:
            uncovered_end = entry_start
    return (uncovered_start, uncovered_end)
def transcribe_audio_file(audio_path=None, json_data=None,video_path=None, chunk_length_ms=60000,renew=False):
    logger.info(f"transcribe_audio_file: {audio_path}")
    json_data = json_data or {}
    audio_path = audio_path or json_data.get('audio_path')
    video_path = video_path or json_data.get('video_path')
    json_data = create_key_value(json_data, 'video_path', video_path)
    file_path = json_data.get('info_path') or os.path.join(os.getcwd(),'info.json')
    json_data = create_key_value(json_data, 'info_path', file_path)
    video_directory = json_data.get('video_directory') or os.path.dirname(file_path)
    json_data = create_key_value(json_data, 'video_directory', video_directory)
    json_data = create_key_value(json_data, 'audio_text', [])
    if audio_path == None:
        if video_path == None:
            logger.info(f"no audio_path and no video_path: {output_path}")
            return json_data
        audio_path = os.path.join(video_directory,'audio.wav')
        json_data = create_key_value(json_data, 'audio_path', audio_path)
    if not os.path.isfile(audio_path):
        extract_audio_from_video(video_path, audio_path)
    temp_file_path = file_path.replace('.json', '_temp.json')
    full_text = ""
    chunk_path = os.path.join(video_directory,"temp_chunk.wav")
    audio = AudioSegment.from_wav(audio_path)
    chunks = [audio[i:i + chunk_length_ms] for i in range(0, len(audio), chunk_length_ms)]
    
    for i, chunk in enumerate(chunks):
        start_time = i * chunk_length_ms
        end_time = min((i + 1) * chunk_length_ms, len(audio))
        result = check_if_transcribed_block(json_data['audio_text'], start_time, end_time)
        if result is True and renew == False:
            logger.info(f"block {start_time} - {end_time} is already transcribed")
            continue
        else:
            logger.info(f"transcribing block {start_time} - {end_time}")
            if result is True:
                result = start_time,end_time
            updated_start, updated_end = result
            chunk_path = f"{video_directory}/chunk_{i}_partial.wav"
            audio[updated_start:updated_end].export(chunk_path, format="wav")
            with sr.AudioFile(chunk_path) as source:
                r.adjust_for_ambient_noise(source)
                audio_data = r.record(source)
                try:
                    text = r.recognize_google(audio_data)
                except:
                    text = ""
            
            json_data['audio_text'].append({
                "start_time": format_timestamp(updated_start),
                "end_time": format_timestamp(updated_end),
                "text": text
            })
            safe_dump_to_file(json_data, temp_file_path)
        if os.path.isfile(chunk_path):
            os.remove(chunk_path)
    if os.path.isfile(chunk_path):
        os.remove(chunk_path)
    if os.path.isfile(temp_file_path):
        shutil.move(temp_file_path,file_path)
    return json_data
def export_srt(audio_text, output_path):
    logger.info(f"export_srt: {output_path}")
    with open(output_path, 'w') as f:
        for i, entry in enumerate(audio_text, 1):
            start = entry['start_time'].replace('.', ',')
            end = entry['end_time'].replace('.', ',')
            f.write(f"{i}\n{start} --> {end}\n{entry['text']}\n\n")
def extract_audio_from_video(video_path, audio_path):
    """Extract audio from a video file using moviepy."""
    try:
        logger.info(f"Extracting audio from {video_path} to {audio_path}")
        video = mp.VideoFileClip(video_path)
        video.audio.write_audiofile(audio_path)
        video.close()
        if not os.path.exists(audio_path):
            raise FileNotFoundError(f"Audio file {audio_path} was not created.")
        logging.info(f"Audio extracted successfully: {audio_path}")
        return audio_path
    except Exception as e:
        logging.error(f"Error extracting audio from {video_path}: {e}")
        return None
def get_voice(voice, text=None):
    """Save transcribed text to a file."""
    text = text or ''
    if voice:
        text = text + '\n' + str(voice) if text else str(voice)
    return text
